#include <stdio.h>
#include <stdlib.h>

#include "LPost.h"
#include "lex.h"
#include "darray.h"
#include "useful.h"
#include "memory.h"
#include "tagger.h"

#define BUF_SIZE 10240

Registry lexicon_hash, good_right_hash, good_left_hash;
Registry seenTagging_hash;

Darray lRuleArray,cRuleArray,bigramArray;

/* initializes the tagger by loading all relevant data files, given a
   base path to the location of those files.  Note, this means that
   the calling Perl side must supply the base path. */
void _initialize(char *basepath) {
  FILE *lexicon, *bigrams, *lRuleFile, *cRuleFile;
  char *filename = malloc(strlen(basepath)+80);
  int i=0;

  sprintf(filename, "%s/LEXICON", basepath);
  lexicon = fopen(filename, "r");
  sprintf(filename, "%s/BIGRAMS", basepath);
  bigrams = fopen(filename, "r");
  sprintf(filename, "%s/LEXICALRULEFILE", basepath);
  lRuleFile = fopen(filename, "r");
  sprintf(filename, "%s/CONTEXTUALRULEFILE", basepath);
  cRuleFile = fopen(filename, "r");

  if ( lexicon == NULL || bigrams == NULL || 
       lRuleFile == NULL || cRuleFile == NULL ) {
    printf("Datafiles missing!\n");
    return;
  }

  Tagger (lexicon, bigrams, lRuleFile, cRuleFile,
	  &lexicon_hash, &good_right_hash,
	  &good_left_hash, &seenTagging_hash,
	  &bigramArray, &lRuleArray, &cRuleArray);

  fclose(cRuleFile);
  fclose(lRuleFile);
  fclose(bigrams);
  fclose(lexicon);
  free(filename);

  return;
}

/* tags a string.  Note that the code seg faults if it hasn't been
   properly initialized. */
char *tag(char *buf) {
  int numSen = 1;
  char **buffer = (char **) malloc(sizeof(char *));
  int i;

  buffer[0] = (char *) malloc((strlen(buf)+1)*sizeof(char *));
  strcpy(buffer[0], buf);

  buffer = StartStateTagger (buffer, numSen, lexicon_hash,
			     good_right_hash, good_left_hash,
			     bigramArray, lRuleArray);

  /* lexicon_hash is effectively the same as a word_hash, which
     is read from the lexicon file and originally used here. We
     just replace word_hash with lexicon_hash here (save memory!) */
  buffer = FinalStateTagger (buffer, numSen, seenTagging_hash,
			     lexicon_hash, cRuleArray);
  
  return buffer[0]; 
}

/* performs general cleanup */
void destroy_tagger(void) {
  int i;

  Registry_traverse(lexicon_hash,FreeRegistryEntry,NULL);
  Registry_destroy(lexicon_hash);

  Registry_traverse(good_right_hash,FreeRegistryEntry,NULL);
  Registry_destroy(good_right_hash);

  Registry_traverse(good_left_hash,FreeRegistryEntry,NULL);
  Registry_destroy(good_left_hash);

  Registry_traverse(seenTagging_hash,FreeRegistryEntry,NULL);
  Registry_destroy(seenTagging_hash);

  for (i=0;i<Darray_len(lRuleArray);++i)
    free((char*)Darray_get(lRuleArray,i));

  for (i=0;i<Darray_len(cRuleArray);++i)
    free((char*)Darray_get(cRuleArray,i));

  for (i=0;i<Darray_len(bigramArray);++i)
    free((char*)Darray_get(bigramArray,i));

  Darray_destroy(lRuleArray);
  Darray_destroy(cRuleArray);
  Darray_destroy(bigramArray);
}
